package multiava.gfx;

import java.nio.DoubleBuffer;

import javax.media.opengl.GL;

import com.emarcotte.glfoo.GraphicsDriver;
import com.emarcotte.glfoo.Renderable;
import com.emarcotte.glfoo.utils.GLUtil;
import com.sun.opengl.util.BufferUtil;

/**
 * Initial attempt at a wrapper for various VBO bits.
 * 
 * Eventually it should be possible to do most operations through this object.
 * 
 * @author Eugene Marcotte <emarcotte@gmail.com>
 */
public class VertexBufferObject implements Renderable {
	private int positionBuffer;
	private int texCoordBuffer;
	private int normalBuffer;
	DoubleBuffer verts;
	DoubleBuffer texCoords;
	DoubleBuffer normals;
	int faceMode, vertCount;

	public void setNormals(double[] n) {
		normals = DoubleBuffer.wrap(n);
	}

	public void setTextureCoordinates(double[] t) {
		texCoords = DoubleBuffer.wrap(t);
	}

	public void setVertexPositions(double[] v, int faceMode) {
		if (faceMode == GL.GL_TRIANGLES) {
			this.faceMode = faceMode;
		} else if (faceMode == GL.GL_QUADS) {
			this.faceMode = faceMode;
		} else {
			throw new IllegalArgumentException("Unsupported face mode");
		}
		verts = DoubleBuffer.wrap(v);
		vertCount = v.length / 3;
	}

	@Override
	public void draw(GraphicsDriver driver) {
		GL gl = driver.getGl();

		gl.glEnableClientState(GL.GL_VERTEX_ARRAY);
		gl.glBindBufferARB(GL.GL_ARRAY_BUFFER, positionBuffer);
		gl.glVertexPointer(3, GL.GL_DOUBLE, 0, 0);

		if (texCoords != null) {
			gl.glEnableClientState(GL.GL_TEXTURE_COORD_ARRAY);
			gl.glBindBufferARB(GL.GL_ARRAY_BUFFER, texCoordBuffer);
			gl.glTexCoordPointer(2, GL.GL_DOUBLE, 0, 0);
		}

		if (normals != null) {
			gl.glEnableClientState(GL.GL_NORMAL_ARRAY);
			gl.glBindBufferARB(GL.GL_ARRAY_BUFFER, normalBuffer);
			gl.glNormalPointer(GL.GL_DOUBLE, 0, 0);
		}

		gl.glDrawArrays(faceMode, 0, vertCount);

		gl.glDisableClientState(GL.GL_NORMAL_ARRAY);
		gl.glDisableClientState(GL.GL_TEXTURE_COORD_ARRAY);
		gl.glDisableClientState(GL.GL_VERTEX_ARRAY);
	}

	@Override
	public void init(GraphicsDriver driver) {
		GL gl = driver.getGl();
		int buffer = 0;
		int[] buffers = GLUtil.genBuffers(gl, getNumberOfBuffersNeeded());
		positionBuffer = buffers[buffer];
		buffer++;

		gl.glBindBufferARB(GL.GL_ARRAY_BUFFER, positionBuffer);
		gl.glBufferDataARB(GL.GL_ARRAY_BUFFER, verts.capacity()
				* BufferUtil.SIZEOF_DOUBLE, verts, GL.GL_STREAM_DRAW);

		if (texCoords != null) {
			texCoordBuffer = buffers[buffer];
			buffer++;
			gl.glBindBufferARB(GL.GL_ARRAY_BUFFER, texCoordBuffer);
			gl.glBufferDataARB(GL.GL_ARRAY_BUFFER, texCoords.capacity()
					* BufferUtil.SIZEOF_DOUBLE, texCoords, GL.GL_STREAM_DRAW);
		}

		if (normals != null) {
			normalBuffer = buffers[buffer];
			buffer++;
			gl.glBindBufferARB(GL.GL_ARRAY_BUFFER, normalBuffer);
			gl.glBufferDataARB(GL.GL_ARRAY_BUFFER, normals.capacity()
					* BufferUtil.SIZEOF_DOUBLE, normals, GL.GL_STREAM_DRAW);
		}
	}

	public int getNumberOfBuffersNeeded() {
		int buffers = 1;
		if (normals != null) {
			buffers++;
		}
		if (texCoords != null) {
			buffers++;
		}
		return buffers;
	}
}
